#!/bin/sh

# Copyright (C) 2023  Braiins Systems s.r.o.
#
# This file is part of Braiins Open-Source Initiative (BOSI).
#
# BOSI is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# Please, keep in mind that we may also license BOSI or any part thereof
# under a proprietary license. For more information on the terms and conditions
# of such proprietary license or if you have any other questions, please
# contact us at opensource@braiins.com.

stop_and_kill_service() {
    service_name=$1
    process_name=$2
    /etc/init.d/$service_name stop &
    sleep 1
    killall -9 $process_name 2> /dev/null || true
}

set -e -o pipefail

ROOT="$PWD"
BASEDIR=$(dirname "$0")
REQUIRED_FREE_SPACE_TMP=30000
BOS_TOOLS_NAME="$BASEDIR/bos-tools"
BOS_BOOT_FILE="$BASEDIR/openwrt-stm32mp15-ii1-emmc-mmcblk0bootx.img"
BOS_MMCBLK0_FILE="$BASEDIR/openwrt-stm32mp15-ii1-emmc-mmcblk0.img"
BOS_ROOTFS_FILE="$BASEDIR/openwrt-stm32mp15-ii1-emmc-squashfs-mmcblk0gp0.img"
REQUIRED_BOS_VERSION="2023-07-23-0-7b66080f-23.08-plus-nightly"

# Using hidden `--disconnect-and-update` parameter will exclude checks, and only write to flash
# This feature is being used to perform forking to background, so when SSH connection is interrupted
if [ "$1" != "--disconnect-and-update" ]; then
    gzip -d $BASEDIR/*.gz
    # Check if system is suitable for BOS
    # Check the existence and functionality of all required system utilities
    for i in dd uname grep awk df; do
        if ! [ -x "$(command -v $i)" ]; then
          echo "Error: $i is not installed." >&2
          exit 1
        fi
    done

    # Check that required files are available
    for i in $BOS_BOOT_FILE $BOS_MMCBLK0_FILE $BOS_ROOTFS_FILE $BOS_TOOLS_NAME; do
        if [ ! -f $i ]; then
            echo "Error: Required file $i for installation is missing." >&2
            exit 1
        fi
    done

    if ! grep -q STM32 "/proc/cpuinfo"; then
        echo "Error: This install script is only for STM32." >&2
        exit 1
    fi

    if ! grep -q "$REQUIRED_BOS_VERSION" "/etc/bos_version"; then
        echo "Error: This install script is only for BOS version $REQUIRED_BOS_VERSION." >&2
        exit 1
    fi

    # Check architecture
    if ! uname -a | grep -q armv7l; then
        echo "Error: This install script is only for armv7l." >&2
        exit 1
    fi

    if ! grep -q -x "emmc" /etc/bos_mode; then
        echo "Error: This install script is only for eMMC." >&2
        exit 1
    fi

    TMP_FREE_SPACE=$(df | grep 'tmpfs.*/tmp' | awk '{print $4}')
    if [ $REQUIRED_FREE_SPACE_TMP -gt "$TMP_FREE_SPACE" ];  then
        echo "Error: Insufficient space on /tmp." >&2
        exit 1
    fi

    # Stop daemons
    stop_and_kill_service uhttpd uhttpd
    stop_and_kill_service dnsmasq dnsmasq

    cd "$ROOT"
    $BOS_TOOLS_NAME daemonize /bin/sh -- -c "source $0" "$0" "--disconnect-and-update" 2>&1 0</dev/null

    exit 0
fi

# This part of install script is run on background

# Redirect the stdout to /dev/kmsg because we need to disconnect from
# first part of the script
echo on > /proc/sys/kernel/printk_devkmsg
exec &>/dev/kmsg

# Terminate dropbear
stop_and_kill_service dropbear dropbear

# Flash eMMC
# Enable write access
echo 0 > /sys/class/block/mmcblk0boot0/force_ro
echo 0 > /sys/class/block/mmcblk0boot1/force_ro

# <send_ack> =1 to enable the boot acknowledge bit in the eMMC ext_csd register
mmc bootbus set single_backward x1 x1 /dev/mmcblk0

# WARNING: Erase the entire 'fip' partition if it was previously flashed!
dd if=/dev/zero of=/dev/mmcblk0 bs=4M count=4 conv=fsync

# Write images
dd if="$BOS_BOOT_FILE" of=/dev/mmcblk0boot0 bs=4096 conv=fsync
dd if="$BOS_BOOT_FILE" of=/dev/mmcblk0boot1 bs=4096 conv=fsync

mmc bootpart enable 1 1 /dev/mmcblk0

dd if="$BOS_MMCBLK0_FILE" of=/dev/mmcblk0 bs=4096 conv=fsync
dd if="$BOS_ROOTFS_FILE" of=/dev/mmcblk0gp0 bs=4M conv=fsync

sync

reboot -f
sleep 5
echo b 2> /dev/null > /proc/sysrq-trigger
